module post2d_extr_mod
  use sep
  implicit none
  real,    private :: ta,tb
  logical, private :: stable
contains

  subroutine post2d_param()

    call from_param("stable",stable,.false.)
    call from_param("a",ta,0.47)
    call from_param("b",tb,0.37)

  end subroutine post2d_param

  subroutine post2d_phase_shift(wfld,wv0,dz,dx,nx)
    complex :: wfld(:)
    real:: wv0,dz,dx
    integer ::nx

    real :: phsft,kx,fkx,dkx,kz2,pi
    integer :: ix

    pi=3.1415926
    fkx=-pi/dx
    dkx=2.0*pi/(nx*dx)

    do ix=1,nx
       kx=fkx+(ix-1)*dkx
       kz2=1.0-1.0/(wv0*wv0)*(kx*kx)
       if (kz2 .le. 0.0 ) then !??????
          wfld(ix)=0.0
       else
          phsft=-dz*wv0*(sqrt(kz2))
          wfld(ix)=wfld(ix)*cmplx(cos(phsft),sin(phsft))
       end if

    end do

  end subroutine post2d_phase_shift


  subroutine post2d_ssf(wfld,wvel,wv0,dz,nx) 
    complex :: wfld(:)
    real    :: wv0,phsft,dz
    real    :: wvel(:)
    integer :: ix,nx

    do ix=1,nx
       phsft=-dz*(wvel(ix)-wv0)
       wfld(ix)=wfld(ix)*cmplx(cos(phsft),sin(phsft))
    end do

  end subroutine post2d_ssf

  subroutine post2d_fd(wfld,v,w,dz,dx,nx)
    implicit none
    complex :: wfld(:)
    real    :: v(:),dz,dx,w
    integer :: nx

    integer :: ix
    complex, allocatable :: u(:)
    complex, allocatable :: a(:),b(:),c(:),l(:),r(:),rr(:)
    real :: lr,li,phsft
!    logical :: stable


!    call from_param("stable",stable,.true.)
!
!    call from_param("a",ta,0.47)
!    call from_param("b",tb,0.37)

    allocate (u(nx),a(nx),l(nx),r(nx),b(nx-1),c(nx-1),rr(nx))
    do ix=1,nx
       phsft=-w*dz/v(ix)
       wfld(ix)=wfld(ix)*cmplx(cos(phsft), sin(phsft))
    end do
    if ( .NOT. stable) then
       do ix=1,nx
          lr=+tb*(v(ix)*v(ix))/(w*w*dx*dx)
          li=ta*dz*v(ix)/(2.0*dx*dx*w)
          l(ix)=cmplx(lr,li)
          r(ix)=cmplx(lr,-li)
          a(ix)=1.0-2.0*l(ix)
          if (ix.le.nx-1) then
             b(ix)=l(ix)
          end if
          if (ix.ge.2) then
             c(ix-1)=l(ix)
          end if
       end do

       do ix=2,nx-1
          u(ix)=(r(ix)*wfld(ix-1)+r(ix)*wfld(ix+1)) +(1-2.0*r(ix))*wfld(ix)
       end do
       ix=1
       u(ix)=(1-2.0*r(ix))*wfld(ix)+r(ix)*wfld(ix+1)
       ix=nx
       u(ix)=(1-2.0*r(ix))*wfld(ix)+r(ix)*wfld(ix-1)

    else
       do ix=1,nx
          lr=+tb*(v(ix)*v(ix))/(w*w*dx*dx)
          li=ta*dz*v(ix)/(2.0*dx*dx*w)
          l(ix)=cmplx(lr,li)
          rr(ix)=cmplx(lr,-li)
          a(ix)=1.0-2.0*l(ix) 
          if (ix.le.nx-1) then
             lr=+tb*(v(ix)*v(ix+1))/(w*w*dx*dx)
             li= ta*dz*v(ix)/(2.0*dx*dx*w)
             b(ix)=cmplx(lr,li)
             l(ix)=cmplx(lr,-li)
          end if
          if (ix .ge. 2) then
             lr=+tb*(v(ix)*v(ix-1))/(w*w*dx*dx)
             li= ta*dz*v(ix)/(2.0*dx*dx*w)
             c(ix-1)=cmplx(lr,li)
             r(ix)=cmplx(lr,-li)
          end if
       end do


       do ix=2,nx-1
          u(ix)=(l(ix)*wfld(ix+1)+r(ix)*wfld(ix-1)) +(1-2.0*rr(ix))*wfld(ix)
       end do
       ix=1
       u(ix)=(1-2.0*rr(ix))*wfld(ix)+l(ix)*wfld(ix+1)
       ix=nx
       u(ix)=(1-2.0*rr(ix))*wfld(ix)+r(ix)*wfld(ix-1)

    end if
    call thr(a,b,c,u,nx)

    wfld=u
    deallocate(a,b,c,u,l,r,rr)

  end subroutine post2d_fd


  subroutine post2d_ffd(wfld,v,vr,w,dz,dx,nx)
    implicit none
    complex :: wfld(:)
    real    :: v(:),vr,dz,dx,w
    integer :: nx

    integer :: ix
    complex, allocatable :: u(:)
    complex, allocatable :: a(:),b(:),c(:),l(:),r(:),rr(:)
    real :: lr,li,phsft
!    logical::stable
!
!    call from_param("stable",stable,.true.)
!
!    call from_param("a",ta,0.47)
!    call from_param("b",tb,0.37)
!

    allocate (u(nx),a(nx),l(nx),r(nx),b(nx-1),c(nx-1),rr(nx))
!write(0,*) ta,tb



    if ( .NOT. stable) then
       do ix=1,nx
          lr=tb*(v(ix)*v(ix)+v(ix)*vr+vr*vr)/(w*w*dx*dx)
          li=ta*dz*(v(ix)-vr)/(2.0*dx*dx*w)
          l(ix)=cmplx(lr,li)
          r(ix)=cmplx(lr,-li)
          a(ix)=1.0-2.0*l(ix)
          if (ix.le.nx-1) then
             b(ix)=l(ix)
          end if
          if (ix.ge.2) then
             c(ix-1)=l(ix)
          end if
       end do

       do ix=2,nx-1
          u(ix)=r(ix)*(wfld(ix-1)+wfld(ix+1)) +(1-2.0*r(ix))*wfld(ix)
       end do
       ix=1
       u(ix)=(1-2.0*r(ix))*wfld(ix)+r(ix)*wfld(ix+1)
       ix=nx
       u(ix)=(1-2.0*r(ix))*wfld(ix)+r(ix)*wfld(ix-1)
    else
       do ix=1,nx
          !lr=tb*(v(ix)*v(ix)+v(ix)*vr+vr*vr)/(w*w*dx*dx)
          !!lr=tb*(v(ix)*v(ix))/(w*w*dx*dx)
          !li=ta*dz*(v(ix)-vr)/(2.0*dx*dx*w)
          !!li=lr
          !!li=ta*dz*v(ix)/(2.0*dx*dx*w)

          !l(ix)=cmplx(lr,li)
          !rr(ix)=cmplx(lr,-li)


          !a(ix)=1.0-2.0*l(ix)      
          if (ix .le.nx-1 .and. ix .ge. 2) then
             lr=tb*(0.25*(v(ix+1)+2*v(ix)+v(ix-1))*0.25*(v(ix+1)+2*v(ix)+v(ix-1))+vr*vr+vr*0.25*(v(ix+1)+2*v(ix)+v(ix-1)))/(w*w*dx*dx)
             li=ta*dz*( 0.25*(v(ix+1)+2*v(ix)+v(ix-1))-vr   )/(2.0*dx*dx*w)
          else
             lr=tb*(v(ix)*v(ix)+vr*vr+vr*v(ix))/(w*w*dx*dx)
             li=ta*dz*(v(ix)-vr)/(2.0*dx*dx*w)
          end if
          l(ix)=cmplx(lr,li)
          rr(ix)=cmplx(lr,-li)
          a(ix)=1.0-2.0*l(ix)

          if (ix .le. nx-1) then
             !lr=tb*(v(ix)*v(ix+1)+sqrt(v(ix)*v(ix+1))*vr+vr*vr)/(w*w*dx*dx)
             !!lr=tb*(v(ix)*v(ix+1))/(w*w*dx*dx)
             !li=ta*dz*( sqrt(v(ix)*v(ix))-vr)/(2.0*dx*dx*w)
             !!li=lr
             !!li=ta*dz*v(ix)/(2.0*dx*dx*w)
             !b(ix)=cmplx(lr,li)
             !l(ix)=cmplx(lr,-li)
             lr=tb*(0.5*(v(ix)+v(ix+1))*0.5*(v(ix)+v(ix+1))+vr*vr+vr*0.5*(v(ix)+v(ix+1)))/(w*w*dx*dx)
             li=ta*dz*(0.5*(v(ix)+v(ix+1))-vr)/(2.0*dx*dx*w)
             b(ix)=cmplx(lr,li)
             l(ix)=cmplx(lr,-li)
          end if
          if (ix .ge. 2) then
             !lr=tb*(v(ix)*v(ix-1)+sqrt(v(ix)*v(ix-1))*vr+vr*vr)/(w*w*dx*dx) 
             !!lr=tb*(v(ix)*v(ix-1))/(w*w*dx*dx)
             !li=ta*dz*( sqrt(v(ix)*v(ix))-vr)/(2.0*dx*dx*w)
             !!li=lr
             !!li=ta*dz*v(ix)/(2.0*dx*dx*w)

             !c(ix-1)=cmplx(lr,li)
             !r(ix)=cmplx(lr,-li)
             lr=tb*(0.5*(v(ix)+v(ix-1))*0.5*(v(ix)+v(ix-1))+vr*vr+vr*0.5*(v(ix)+v(ix-1)))/(w*w*dx*dx)
             li=ta*dz*(0.5*(v(ix)+v(ix-1))-vr)/(2.0*dx*dx*w)
             c(ix-1)=cmplx(lr,li)
             r(ix)=cmplx(lr,-li)
          end if
       end do

       do ix=2, nx-1
          u(ix)=(l(ix)*wfld(ix+1)+r(ix)*wfld(ix-1))+(1-2.0*rr(ix))*wfld(ix)
       end do
       ix=1
       u(ix)=(1-2.0*rr(ix))*wfld(ix)+l(ix)*wfld(ix+1)
       ix=nx
       u(ix)=(1-2.0*rr(ix))*wfld(ix)+r(ix)*wfld(ix-1)
    end if

    call thr(a,b,c,u,nx)

    wfld=u
    deallocate(a,b,c,u,l,r,rr)

  end subroutine post2d_ffd


  subroutine thr(a,b,c,v,n)
    implicit none
    integer n
    complex a(n),b(n-1),c(n-1),v(n)
    integer i
    b(1)=b(1)/a(1)
    v(1)=v(1)/a(1)
    do i=2,n
       a(i)=a(i)-c(i-1)*b(i-1)
       b(i)=b(i)/a(i)
       v(i)=(v(i)-c(i-1)*v(i-1))/a(i)
    enddo
    do i=n-1,1,-1
       v(i)=v(i)-b(i)*v(i+1)
    enddo
    return
  end subroutine thr


end module post2d_extr_mod
