module post2d_mig
  use post2d_extr_mod
  use sep3d_struct_mod
  use sep

  implicit none
  type(sep3d):: velocity
  type(sep3d):: data
  integer :: nt,nx,nz,iw1,iw2,nw
  real    :: dt,dx,dz,dw,oz,ox,ow
  real, allocatable    :: vel_block(:,:)
  complex, allocatable    ::wfld_all(:,:)
contains

  subroutine post2d_mig_init()
    implicit none
    real :: pi,f1,f2,x
    real, allocatable :: tempr(:),vel(:,:)
    complex, allocatable :: tmprec(:)
    integer :: ix,it,vel_ix,ierr
    integer,external::auxin,sreed


    call from_param("min_f",f1,5.0)
    call from_param("max_f",f2,50.0)
    pi=3.1415926


    if ( -1==auxin("velocity") ) then
       write(0,*) "Must provide velocity file"
       return
    end if
    call init_sep3d("velocity",velocity,"INPUT")
    if (.not. valid_structure(velocity)) then
       write(0,*) "invalid velocity init"
       return
    end if

    call init_sep3d("in",data,"INPUT")

    dw=data%d(2) *2*pi
    nw=data%n(2)
    ow=data%o(2) *2*pi
    dx=data%d(1)
    nx=data%n(1)
    ox=data%o(1)

    oz=velocity%o(1)
    dz=velocity%d(1)
    nz=velocity%n(1)


    allocate( vel(velocity%n(1),velocity%n(2)))
    if (sreed("velocity",vel,4*velocity%n(1)*velocity%n(2))/=4*velocity%n(1)*velocity%n(2) ) call seperr("err reading data")
    vel=vel*0.5

    allocate(vel_block(nz,nx))
    !vel_block
    do ix=1,data%n(1)
       x=(ix-1)*data%d(1)+data%o(1)
       vel_ix=(x-velocity%o(2))/velocity%d(2)+1
       if (vel_ix <1) then
          vel_block(:,ix)=vel(:,1)
       end if
       if (vel_ix >velocity%n(2) ) then
          vel_block(:,ix)=vel(:,velocity%n(2))
       end if
       if (vel_ix >=1 .and. vel_ix <= velocity%n(2)) then
          vel_block(:,ix)=vel(:,vel_ix)
       end if
    end do
    deallocate(vel)


    allocate( wfld_all(nx,nw),stat=ierr )
    if (sreed("in",wfld_all,8*nx*nw)/=8*nx*nw ) call seperr("error reading")

  end subroutine post2d_mig_init






  subroutine post2d_down()
    integer :: iw,iz,ix
    real    :: w,wv0,vr
    real,allocatable    :: wvel(:),v0(:)
    complex,allocatable :: wfld(:)
    real, allocatable :: image(:,:),tempr(:)

    integer :: st,ierr
    integer, external ::srite,putch
    logical :: ffd,ssf,fd
    real ::add


    call from_param("ssf",ssf,.true.)
    call from_param("ffd",ffd,.true.)
    call from_param("fd",fd,.false.)
    call from_param("vr",vr,10.0)
    allocate(image(nz,nx))
    allocate(wvel(nx),v0(nz))
    allocate(wfld(nx))



    do iz=1,nz
       v0(iz)=minval(vel_block(iz,:))
       !   v0(iz)=maxval(vel_block(iz,:))
    end do

    do iz=1,nz
       add=0
       do ix=1,nx
          add=add+vel_block(iz,ix)
       end do
       add=add/nx
       ! v0(iz)=0.0*add+1.0*v0(iz)

    end do

    call post2d_param()
    image=0.0
    do iw=1,nw
       write(0,*) iw,nw, minval(image),maxval(image)
       w=ow+(iw-1)*dw
       wfld=wfld_all(:,iw)
       do iz=1,nz
          if (.not. fd ) then
             call FFT1D(wfld,nx,+1)
             wv0=w/v0(iz)
             call post2d_phase_shift(wfld,wv0,dz,dx,nx)
             call FFT1D(wfld,nx,-1)
             wvel=vel_block(iz,:)
             wvel=w/wvel
             if ( ssf ) then
                call post2d_ssf(wfld,wvel,wv0,dz,nx)
             end if
             if ( ffd .AND. ssf ) then 
                call post2d_ffd(wfld,vel_block(iz,:),v0(iz),w,dz,dx,nx)
             end if
          else
             wvel=vel_block(iz,:)
             call post2d_fd(wfld,wvel,w,dz,dx,nx)      
          end if
          do ix=1,nx
             image(iz,ix)=image(iz,ix)+real(wfld(ix))
          end do

       end do
    end do

    allocate(tempr(nx*nz))
    tempr=reshape(image,(/size(tempr)/))
    ierr=srite("out",tempr,4*nx*nz)
    ierr=putch("n1","d",nz)
    ierr=putch("n2","d",nx)
    ierr=putch("d1","f",dz)
    ierr=putch("d2","f",dx)
    ierr=putch("o1","f",oz)
    ierr=putch("o2","f",ox)

    ierr=putch("esize","d",4)

    deallocate( image,wvel,v0,wfld,tempr,vel_block)
  end subroutine post2d_down


end module post2d_mig
